@@ -6,15 +6,16 @@ import validations
 # For gettext
 N_ = lambda x: x
 
-NOTE_EXTENSIONS = N_("File extension list to which content the configuration will be applied.")
+NOTE_EXTENSIONS = N_("Comma-separated list of File Extension to which the configuration will be applied.")
 
 class ModuleExtensions (Module, FormHelper):
-    validation = [('tmp!new_rule!value', validations.is_safe_id_list)]
-
     def __init__ (self, cfg, prefix, submit_url):
         FormHelper.__init__ (self, 'extensions', cfg)
         Module.__init__ (self, 'extensions', cfg, prefix, submit_url)
 
+        self.validation = [('tmp!new_rule!value',     validations.is_safe_id_list),
+                           ('%s!extensions'%(prefix), validations.is_safe_id_list)]
+
     def _op_render (self):
         table = TableProps()
         if self._prefix.startswith('tmp!'):
