@@ -27,7 +27,11 @@ DATA_VALIDATION = [
     ("server!iocache!min_file_size",  validations.is_positive_int),
     ("server!iocache!max_file_size",  validations.is_positive_int),
     ("server!iocache!lasting_stat",   validations.is_positive_int),
-    ("server!iocache!lasting_mmap",   validations.is_positive_int)
+    ("server!iocache!lasting_mmap",   validations.is_positive_int),
+    ("server!tls!dh_param512",       (validations.is_local_file_exists, 'cfg')),
+    ("server!tls!dh_param1024",      (validations.is_local_file_exists, 'cfg')),
+    ("server!tls!dh_param2048",      (validations.is_local_file_exists, 'cfg')),
+    ("server!tls!dh_param4096",      (validations.is_local_file_exists, 'cfg'))
 ]
 
 WARNING = N_('<p><b>WARNING</b>: This section contains advanced configuration parameters. Changing things is not recommended unless you really know what you are doing.</p>')
@@ -51,8 +55,12 @@ NOTE_IO_ENABLED   = N_('Activate or deactivate the I/O cache globally.')
 NOTE_IO_SIZE      = N_('Number of pages that the cache should handle.')
 NOTE_IO_MIN_SIZE  = N_('Files under this size will not be cached.')
 NOTE_IO_MAX_SIZE  = N_('Files over this size will not be cached.')
-NOTE_IO_LAST_STAT = N_('How long the file information should last cached without refreshing it.')
-NOTE_IO_LAST_MMAP = N_('How long the file content should last cached.')
+NOTE_IO_LAST_STAT = N_('How long (in seconds) the file information should last cached without refreshing it.')
+NOTE_IO_LAST_MMAP = N_('How long (in seconds) the file content should last cached.')
+NOTE_DH512        = N_('Path to a Diffie Hellman (DH) parameters PEM file: 512 bits.')
+NOTE_DH1024       = N_('Path to a Diffie Hellman (DH) parameters PEM file: 1024 bits.')
+NOTE_DH2048       = N_('Path to a Diffie Hellman (DH) parameters PEM file: 2048 bits.')
+NOTE_DH4096       = N_('Path to a Diffie Hellman (DH) parameters PEM file: 4096 bits.')
 
 HELPS = [('config_advanced', N_('Advanced'))]
 
@@ -68,6 +76,21 @@ class PageAdvanced (PageMenu, FormHelper):
         self.AddMacroContent ('content', content)
         return Page.Render(self)
 
+    def _render_content (self):
+        tabs  = []
+        tabs += [(_('Connections'),   self._render_performance())]
+        tabs += [(_('Resources'),     self._render_resources())]
+        tabs += [(_('I/O cache'),     self._render_iocache())]
+        tabs += [(_('Special Files'), self._render_special_files())]
+        tabs += [(_('TLS'),           self._render_tls())]
+
+        txt = "<h1>%s</h1>" % (_('Advanced configuration'))
+        txt += self.Dialog(WARNING, 'warning')
+        txt += self.InstanceTab (tabs)
+
+        form = Form ("/%s" % (self._id), add_submit=False)
+        return form.Render(txt, DEFAULT_SUBMIT_VALUE)
+
     def _render_performance (self):
         polling_methods = []
         for name, desc in POLL_METHODS:
@@ -79,56 +102,49 @@ class PageAdvanced (PageMenu, FormHelper):
         self.AddPropCheck (table, _('Keep Alive'),         'server!keepalive',        True, _(NOTE_KEEPALIVE))
         self.AddPropEntry (table, _('Max keepalive reqs'), 'server!keepalive_max_requests', _(NOTE_KEEPALIVE_RS))
         self.AddPropCheck (table, _('Chunked Encoding'),   'server!chunked_encoding', True, _(NOTE_CHUNKED))
-        self.AddPropOptions_Reload (table, _('Polling Method'), 'server!poll_method',  polling_methods, _(NOTE_POLLING))
-        self.AddPropEntry (table, _('Sendfile min size'),  'server!sendfile_min', _(NOTE_SENDFILE_MIN))
-        self.AddPropEntry (table, _('Sendfile max size'),  'server!sendfile_max', _(NOTE_SENDFILE_MAX))
+        self.AddPropOptions_Reload_Plain (table, _('Polling Method'), 'server!poll_method',  polling_methods, _(NOTE_POLLING))
+        self.AddPropEntry (table, _('Sendfile min size'),  'server!sendfile_min', _(NOTE_SENDFILE_MIN), optional=True)
+        self.AddPropEntry (table, _('Sendfile max size'),  'server!sendfile_max', _(NOTE_SENDFILE_MAX), optional=True)
         return self.Indent(table)
 
     def _render_resources (self):
         table = TableProps()
-        self.AddPropEntry (table, _('Thread Number'),          'server!thread_number',        _(NOTE_THREAD_NUM))
-        self.AddPropOptions_Reload (table, _('Thread Policy'), 'server!thread_policy', THREAD_POLICY, _(NOTE_THREAD))
-        self.AddPropEntry (table, _('File descriptors'),       'server!fdlimit',              _(NOTE_FD_NUM))
-        self.AddPropEntry (table, _('Listening queue length'), 'server!listen_queue',         _(NOTE_LISTEN_Q))
-        self.AddPropEntry (table, _('Reuse connections'),      'server!max_connection_reuse', _(NOTE_REUSE_CONNS))
-        self.AddPropEntry (table, _('Log flush time'),         'server!log_flush_lapse',     _(NOTE_FLUSH_TIME))
-        self.AddPropEntry (table, _('Nonces clean up time'),   'server!nonces_cleanup_lapse',_(NOTE_NONCES_TIME))
+        self.AddPropEntry (table, _('Thread Number'),          'server!thread_number',        _(NOTE_THREAD_NUM), optional=True)
+        self.AddPropOptions_Reload_Plain (table, _('Thread Policy'), 'server!thread_policy', THREAD_POLICY, _(NOTE_THREAD))
+        self.AddPropEntry (table, _('File descriptors'),       'server!fdlimit',              _(NOTE_FD_NUM), optional=True)
+        self.AddPropEntry (table, _('Listening queue length'), 'server!listen_queue',         _(NOTE_LISTEN_Q), optional=True)
+        self.AddPropEntry (table, _('Reuse connections'),      'server!max_connection_reuse', _(NOTE_REUSE_CONNS), optional=True)
+        self.AddPropEntry (table, _('Log flush time'),         'server!log_flush_lapse',     _(NOTE_FLUSH_TIME), optional=True)
+        self.AddPropEntry (table, _('Nonces clean up time'),   'server!nonces_cleanup_lapse',_(NOTE_NONCES_TIME), optional=True)
         return self.Indent(table)
 
     def _render_iocache (self):
         table = TableProps()
         self.AddPropCheck (table, _('Status'),        'server!iocache', True,         _(NOTE_IO_ENABLED))
-        self.AddPropEntry (table, _('Max pages'),     'server!iocache!max_size',      _(NOTE_IO_SIZE))
-        self.AddPropEntry (table, _('File Min Size'), 'server!iocache!min_file_size', _(NOTE_IO_MIN_SIZE))
-        self.AddPropEntry (table, _('File Max Size'), 'server!iocache!max_file_size', _(NOTE_IO_MAX_SIZE))
-        self.AddPropEntry (table, _('Lasting: stat'), 'server!iocache!lasting_stat',  _(NOTE_IO_LAST_STAT))
-        self.AddPropEntry (table, _('Lasting: mmap'), 'server!iocache!lasting_mmap',  _(NOTE_IO_LAST_MMAP))
+        self.AddPropEntry (table, _('Max pages'),     'server!iocache!max_size',      _(NOTE_IO_SIZE), optional=True)
+        self.AddPropEntry (table, _('File Min Size'), 'server!iocache!min_file_size', _(NOTE_IO_MIN_SIZE), optional=True)
+        self.AddPropEntry (table, _('File Max Size'), 'server!iocache!max_file_size', _(NOTE_IO_MAX_SIZE), optional=True)
+        self.AddPropEntry (table, _('Lasting: stat'), 'server!iocache!lasting_stat',  _(NOTE_IO_LAST_STAT), optional=True)
+        self.AddPropEntry (table, _('Lasting: mmap'), 'server!iocache!lasting_mmap',  _(NOTE_IO_LAST_MMAP), optional=True)
         return self.Indent(table)
 
     def _render_special_files (self):
         table = TableProps()
-        self.AddPropEntry (table, _('Panic action'), 'server!panic_action', _(NOTE_PANIC_ACTION))
-        self.AddPropEntry (table, _('PID file'),     'server!pid_file',     _(NOTE_PID_FILE))
+        self.AddPropEntry (table, _('Panic action'), 'server!panic_action', _(NOTE_PANIC_ACTION), optional=True)
+        self.AddPropEntry (table, _('PID file'),     'server!pid_file',     _(NOTE_PID_FILE), optional=True)
         return self.Indent(table)
 
-    def _render_content (self):
-        tabs  = []
-        tabs += [(_('Connections'),   self._render_performance())]
-        tabs += [(_('Resources'),     self._render_resources())]
-        tabs += [(_('I/O cache'),     self._render_iocache())]
-        tabs += [(_('Special Files'), self._render_special_files())]
-
-        txt = "<h1>%s</h1>" % (_('Advanced configuration'))
-        txt += self.Dialog(WARNING, 'warning')
-        txt += self.InstanceTab (tabs)
-
-        form = Form ("/%s" % (self._id), add_submit=False)
-        return form.Render(txt, DEFAULT_SUBMIT_VALUE)
-
+    def _render_tls (self):
+	table = TableProps()
+        self.AddPropEntry (table, _('DH parameters: 512 bits'), 'server!tls!dh_param512', _(NOTE_DH512), optional=True)
+        self.AddPropEntry (table, _('DH parameters: 1024 bits'), 'server!tls!dh_param1024', _(NOTE_DH1024), optional=True)
+        self.AddPropEntry (table, _('DH parameters: 2048 bits'), 'server!tls!dh_param2048', _(NOTE_DH2048), optional=True)
+        self.AddPropEntry (table, _('DH parameters: 4096 bits'), 'server!tls!dh_param4096', _(NOTE_DH4096), optional=True)
+        return self.Indent(table)
 
     def _op_apply_changes (self, uri, post):
-        self.ApplyChanges (['server!keepalive', 
-                            'server!chunked_encoding', 
-                            'server!iocache'], 
+        self.ApplyChanges (['server!keepalive',
+                            'server!chunked_encoding',
+                            'server!iocache'],
                            post, validation = DATA_VALIDATION)
 
