@@ -13,6 +13,7 @@ from Rule import *
 N_ = lambda x: x
 
 NOTE_DOCUMENT_ROOT   = N_('Allows to specify an alternative document root path.')
+NOTE_TIMEOUT         = N_('Apply a custom timeout to the connections matching this rule.')
 NOTE_HANDLER         = N_('How the connection will be handled.')
 NOTE_HTTPS_ONLY      = N_('Enable to allow access to the resource only by https.')
 NOTE_ALLOW_FROM      = N_('List of IPs and subnets allowed to access the resource.')
@@ -25,10 +26,11 @@ The <b>m</b>, <b>h</b>, <b>d</b> and <b>w</b> suffixes are allowed for minutes,
 """)
 
 DATA_VALIDATION = [
-    ("vserver!.*?!rule!(\d+)!document_root",  (validations.is_dev_null_or_local_dir_exists, 'cfg')),
-    ("vserver!.*?!rule!(\d+)!allow_from",      validations.is_ip_or_netmask_list),
-    ("vserver!.*?!rule!(\d+)!rate",            validations.is_number_gt_0),
-    ("vserver!.*?!rule!(\d+)!expiration!time", validations.is_time) 
+    ("vserver!(\d+)!rule!(\d+)!document_root",  (validations.is_dev_null_or_local_dir_exists, 'cfg')),
+    ("vserver!(\d+)!rule!(\d+)!allow_from",      validations.is_ip_or_netmask_list),
+    ("vserver!(\d+)!rule!(\d+)!rate",            validations.is_number_gt_0),
+    ("vserver!(\d+)!rule!(\d+)!timeout",         validations.is_number_gt_0),
+    ("vserver!(\d+)!rule!(\d+)!expiration!time", validations.is_time)
 ]
 
 HELPS = [
@@ -85,10 +87,20 @@ class PageEntry (PageMenu, FormHelper):
         # Check what to do..
         if post.get_val('is_submit'):
             self._op_apply_changes (uri, post)
+            if self.has_errors():
+                return self._op_default (uri)
 
         return self._op_default (uri)
 
     def _op_apply_changes (self, uri, post):
+        # Validate
+        validation = DATA_VALIDATION[:]
+
+        # Validate += rule
+        pre = '%s!match'%(self._conf_prefix)
+        rule = Rule (self._cfg, pre, self.submit_url, self.errors, 0)
+        validation += rule.get_validation()
+
         # Handler properties
         pre = "%s!handler" % (self._conf_prefix)
         self.ApplyChanges_OptionModule (pre, uri, post)
@@ -104,11 +116,11 @@ class PageEntry (PageMenu, FormHelper):
             checks.append ('%s!encoder!%s' % (self._conf_prefix, e))
 
         pre = '%s!match'%(self._conf_prefix)
-        rule = Rule (self._cfg, pre, self.submit_url, 0)
+        rule = Rule (self._cfg, pre, self.submit_url, self.errors, 0)
         checks += rule.get_checks()
 
         # Apply changes
-        self.ApplyChanges (checks, post, DATA_VALIDATION)
+        self.ApplyChanges (checks, post, validation)
 
     def _op_default (self, uri):
         # Render page
@@ -129,7 +141,7 @@ class PageEntry (PageMenu, FormHelper):
 
         # Load the rule plugin
         pre = "%s!match"%(self._conf_prefix)
-        rule = Rule (self._cfg, pre, self.submit_url, 0)
+        rule = Rule (self._cfg, pre, self.submit_url, self.errors, 0)
 
         txt += rule.get_title()
         return txt
@@ -143,12 +155,12 @@ class PageEntry (PageMenu, FormHelper):
 
         # Handler
         table = TableProps()
-        e = self.AddPropOptions_Reload (table, _('Handler'), '%s!handler'%(pre),
-                                        modules_available(HANDLERS), _(NOTE_HANDLER))
+        e = self.AddPropOptions_Reload_Module (table, _('Handler'), '%s!handler'%(pre),
+                                               modules_available(HANDLERS), _(NOTE_HANDLER))
 
         props = self._get_handler_properties()
-        if props and props.show_document_root:
-            self.AddPropEntry (table, _('Document Root'), '%s!document_root'%(pre), _(NOTE_DOCUMENT_ROOT))
+        if not props or props.show_document_root:
+            self.AddPropEntry (table, _('Document Root'), '%s!document_root'%(pre), _(NOTE_DOCUMENT_ROOT), optional=True)
 
         if e:
             tabs += [(_('Handler'), str(table) + e)]
@@ -161,7 +173,7 @@ class PageEntry (PageMenu, FormHelper):
         tabs += [(_('Encoding'), self._render_encoding())]
 
         # Expiration
-        tabs += [(_('Expiration'), self._render_expiration())]
+        tabs += [(_('Time'), self._render_time())]
 
         # Security
         tabs += [(_('Security'), self._render_security())]
@@ -189,7 +201,7 @@ class PageEntry (PageMenu, FormHelper):
     def _render_rule (self):
         txt  = "<h2>%s</h2>" % (_('Matching Rule'))
         pre  = "%s!match"%(self._conf_prefix)
-        rule = Rule (self._cfg, pre, self.submit_url, 0)
+        rule = Rule (self._cfg, pre, self.submit_url, self.errors, 0)
         txt += rule._op_render()
 
         return txt
@@ -198,15 +210,24 @@ class PageEntry (PageMenu, FormHelper):
         txt = ''
 
         table = TableProps()
-        self.AddPropEntry (table, _('Limit traffic to'), '%s!rate'%(self._conf_prefix), _(NOTE_RATE))
+        self.AddPropEntry (table, _('Limit traffic to'), '%s!rate'%(self._conf_prefix), _(NOTE_RATE), optional=True)
 
         txt += "<h2>%s</h2>" % (_('Traffic Shaping'))
         txt += self.Indent(table)
         return txt
 
 
-    def _render_expiration (self):
+    def _render_time (self):
         txt = ''
+
+        # Timeout
+        table = TableProps()
+        self.AddPropEntry (table, _('Timeout'), '%s!timeout'%(self._conf_prefix), _(NOTE_TIMEOUT), optional=True)
+
+        txt += "<h2>%s</h2>" % (_('Connections timeout'))
+        txt += self.Indent(table)
+
+        # Expiration
         pre = "%s!expiration"%(self._conf_prefix)
 
         table = TableProps()
@@ -234,13 +255,13 @@ class PageEntry (PageMenu, FormHelper):
         txt  += "<h2>%s</h2>" % (_('Access Restrictions'))
         table = TableProps()
         self.AddPropCheck (table, _('Only https'), '%s!only_secure'%(pre), False, _(NOTE_HTTPS_ONLY))
-        self.AddPropEntry (table, _('Allow From'),  '%s!allow_from' %(pre), _(NOTE_ALLOW_FROM))
+        self.AddPropEntry (table, _('Allow From'),  '%s!allow_from' %(pre), _(NOTE_ALLOW_FROM), optional=True)
         txt += self.Indent(table)
 
         txt += "<h2>%s</h2>" % (_('Authentication'))
         table = TableProps()
-        e = self.AddPropOptions_Reload (table, _('Validation Mechanism'), '%s!auth'%(pre),
-                                        modules_available(VALIDATORS), _(NOTE_VALIDATOR))
+        e = self.AddPropOptions_Reload_Module (table, _('Validation Mechanism'), '%s!auth'%(pre),
+                                               modules_available(VALIDATORS), _(NOTE_VALIDATOR))
         txt += self.Indent (table)
         txt += e
 
