@@ -24,6 +24,12 @@ class ConfigNode (object):
             return default
         return subcfg.value
 
+    def get_vals (self, keys, default=None):
+        values = []
+        for k in keys:
+            values += [self.get_val(k)]
+        return values
+
     # Get child
     #
     def _getitem_simple (self, key):
@@ -232,6 +238,9 @@ class Config:
     def get_val (self, path, default=None):
         return self.root.get_val (path, default)
 
+    def get_vals (self, paths, default=None):
+        return self.root.get_vals (paths, default)
+
     def __delitem__ (self, path):
         parent, parent_path, child_name = self._get_parent_node (path)
         if parent and parent._child.has_key(child_name):
@@ -251,7 +260,7 @@ class Config:
     # Serialization
     def serialize (self):
         def sorter(x,y):
-            order = ['server', 'vserver', 'source', 'icons', 'mime']
+            order = ['config', 'server', 'vserver', 'source', 'icons', 'mime']
             a = x.split('!')
             b = y.split('!')
             try:
@@ -267,7 +276,7 @@ class Config:
                 return -1
 
             # Sort rules: reverse
-            if ((len(a) > 3) and 
+            if ((len(a) > 3) and
                 (a[0] == b[0] == 'vserver') and
                 (a[1] == b[1]) and
                 (a[2] == b[2] == 'rule')):
@@ -279,7 +288,7 @@ class Config:
 
         tmp = self.root.serialize().split('\n')
         tmp.sort(sorter)
-        return '\n'.join(tmp)
+        return '\n'.join (filter (lambda x: len(x) > 1, tmp))
 
     def save (self):
         # Try to make a copy
