@@ -8,7 +8,7 @@ LOCALEDIR  = "/usr/local/share/locale"
 WWWROOT    = "/var/cherokee"
 SYSCONFDIR = "/etc"
 LOCALSTATE = "/var"
-VERSION    = "0.99.15"
+VERSION    = "0.99.39"
 
 CHEROKEE_SERVER     = join (PREFIX, "sbin/cherokee")
 CHEROKEE_WORKER     = join (PREFIX, "sbin/cherokee-worker")
@@ -21,3 +21,5 @@ CHEROKEE_DATADIR    = join (DATADIR, "cherokee")
 CHEROKEE_DEPSDIR    = join (DATADIR, "cherokee/deps")
 CHEROKEE_CONFDIR    = join (SYSCONFDIR, "cherokee")
 CHEROKEE_VAR_RUN    = join (LOCALSTATE, "run")
+CHEROKEE_RRD_DIR    = join (LOCALSTATE, "db/cherokee/graphs")
+CHEROKEE_GRAPHS_DIR = join (LOCALSTATE, "db/cherokee/graphs/images")
