@@ -10,6 +10,7 @@ import thread
 import signal
 import socket
 import gettext
+import traceback
 
 # Application modules
 #
@@ -30,9 +31,7 @@ from PageNewConfig import *
 from PageAjaxUpdate import *
 from PageInfoSource import *
 from CherokeeManagement import *
-try:
-    from PageWizard import *
-except: pass
+from config_version import *
 
 # Constants
 #
@@ -58,7 +57,9 @@ class Handler(pyscgi.SCGIHandler):
         status  = "200 OK"
         uri     = self.env['REQUEST_URI']
 
-        if not SELECTED_LANGUAGE:
+        # Translation
+        if (not SELECTED_LANGUAGE and
+            self.env.has_key('HTTP_ACCEPT_LANGUAGE')):
             try:
                 langs = self.env['HTTP_ACCEPT_LANGUAGE']
             except:
@@ -77,6 +78,7 @@ class Handler(pyscgi.SCGIHandler):
 
         if page:
             body = page.HandleRequest (uri, Post())
+
             if body and body[0] == '/':
                 self.send ("Status: 302 Moved Temporarily\r\n" + \
                            "Location: %s\r\n\r\n" % (body))
@@ -96,11 +98,10 @@ class Handler(pyscgi.SCGIHandler):
             page = PageAdvanced(cfg)
         elif uri.startswith('/feedback'):
             page = PageFeedback(cfg)
-        elif uri.startswith('/wizard'):
-            page = PageWizard(cfg)
         elif uri == '/vserver' or \
              uri == '/vserver/' or \
-             uri == '/vserver/ajax_update':
+             uri == '/vserver/ajax_update' or\
+             uri.startswith('/vserver/wizard'):
             page = PageVServers(cfg)
         elif uri.startswith('/vserver/'):
             if "/rule/" in uri:
@@ -136,7 +137,7 @@ class Handler(pyscgi.SCGIHandler):
             manager = cherokee_management_get (cfg)
             error = manager.launch()
             if error:
-                page = PageError (cfg, PageError.COULDNT_LAUNCH, error=error)
+                page = PageError_LaunchFail (cfg, error)
             else:
                 body = "/"
         elif uri.startswith('/stop'):
@@ -168,7 +169,14 @@ class Handler(pyscgi.SCGIHandler):
 
         # Execute page
         if page:
-            body = page.HandleRequest(uri, post)
+            try:
+                body = page.HandleRequest(uri, post)
+            except:
+                trace = traceback.format_exc()
+                page = PageInternelError (trace)
+                body = page.HandleRequest (uri, Post())
+                self.send ('Status: 500 Internal Server Error\r\n\r\n' + body)
+                return
 
         # Is it a redirection?
         if body[0] == '/':
@@ -202,7 +210,7 @@ def main():
 
     # Read the arguments
     try:
-        scgi_port = int(sys.argv[1])
+        scgi_port = sys.argv[1]
         cfg_file  = sys.argv[2]
     except:
         print _("Incorrect parameters: PORT CONFIG_FILE")
@@ -217,16 +225,37 @@ def main():
     os.chdir(os.path.abspath(pathname))
 
     # SCGI server
-    srv = pyscgi.ServerFactory (True, handler_class=Handler, host="127.0.0.1", port=scgi_port)
+    if scgi_port.isdigit():
+        srv = pyscgi.ServerFactory (True, handler_class=Handler, host="127.0.0.1", port=int(scgi_port))
+    else:
+        # Remove the unix socket if it already exists
+        try:
+            mode = os.stat (scgi_port)[stat.ST_MODE]
+            if stat.S_ISSOCK(mode):
+                print "Removing an old '%s' unix socket.." %(scgi_port)
+                os.unlink (scgi_port)
+        except OSError:
+            pass
+
+        srv = pyscgi.ServerFactory (True, handler_class=Handler, unix_socket=scgi_port)
+
     srv.socket.settimeout (MODIFIED_CHECK_ELAPSE)
 
     # Read configuration file
     global cfg
     cfg = Config(cfg_file)
 
+    # Update the configuration file if needed
+    config_version_update_cfg (cfg)
+
+    # Let the user know what is going on
     version = VERSION
     pid     = os.getpid()
-    print _("Server %(version)s running.. PID=%(pid)d Port=%(scgi_port)d") % (locals())
+
+    if scgi_port.isdigit():
+        print _("Server %(version)s running.. PID=%(pid)d Port=%(scgi_port)s") % (locals())
+    else:
+        print _("Server %(version)s running.. PID=%(pid)d Socket=%(scgi_port)s") % (locals())
 
     # Iterate until the user exists
     try:
